<?php

namespace jrosset\EnvReader\XmlStructure;

use jrosset\ArrayClasses\ArrayClass;
use jrosset\ArrayClasses\InsensitiveCaseArrayClass;
use SimpleXMLElement;

/**
 * Structure about children nodes
 */
class ChildrenXmlStructure implements IXmlStructure {
    use TAttributeListXmlStructure;

    /**
     * @var ArrayClass The children nodes
     */
    private ArrayClass $childrenNodes;
    /**
     * @var InsensitiveCaseArrayClass Mapping between child node name and properties name
     */
    private ArrayClass $mapping;

    /**
     * Initialize
     */
    public function __construct () {
        $this->childrenNodes = new ArrayClass();
        $this->mapping = new ArrayClass();
    }

    /**
     * Add a child node
     *
     * @param string        $childNodeName      The child node's name
     * @param IXmlStructure $childNodeStructure The child node's structure
     * @param string|null   $propertyName       The corresponding property name. Null if identical to child node's name
     *
     * @return $this
     */
    public function addChildNode (string $childNodeName, IXmlStructure $childNodeStructure, ?string $propertyName = null): self {
        $this->childrenNodes->set($childNodeName, $childNodeStructure);
        if ($propertyName !== null) {
            $this->mapping->set($childNodeName, $propertyName);
        }
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function parseXml (SimpleXMLElement $xmlNode): InsensitiveCaseArrayClass {
        $properties = $this->parseXmlAttributes($xmlNode);

        /** @var IXmlStructure $childNodeStructure */
        foreach ($this->childrenNodes as $childNodeName => $childNodeStructure) {
            if (!isset($xmlNode->$childNodeName)) {
                continue;
            }
            $properties->set(
                $this->mapping->get($childNodeName, false) ?? $childNodeName,
                $childNodeStructure->parseXml($xmlNode->$childNodeName)
            );
        }

        return $properties;
    }
}